<?php
namespace TYPO3\Media\Domain\Model;

/*                                                                        *
 * This script belongs to the TYPO3 Flow package "TYPO3.Media".           *
 *                                                                        *
 * It is free software; you can redistribute it and/or modify it under    *
 * the terms of the GNU General Public License, either version 3 of the   *
 * License, or (at your option) any later version.                        *
 *                                                                        *
 * The TYPO3 project - inspiring people to share!                         *
 *                                                                        */

/**
 * An image interface
 */
interface ImageInterface extends AssetInterface {

	const ORIENTATION_SQUARE = 'square';
	const ORIENTATION_LANDSCAPE = 'landscape';
	const ORIENTATION_PORTRAIT = 'portrait';

	/**
	 * Inset ratio mode: If an image is attempted to get scaled with the size of both edges stated, using this mode will scale it to the lower of both edges.
	 * Consider an image of 320/480 being scaled to 50/50: because aspect ratio wouldn't get hurt, the target image size will become 33/50.
	 */
	const RATIOMODE_INSET = 'inset';

	/**
	 * Outbound ratio mode: If an image is attempted to get scaled with the size of both edges stated, using this mode will scale the image and crop it.
	 * Consider an image of 320/480 being scaled to 50/50: the image will be scaled to height 50, then centered and cropped so the width will also be 50.
	 */
	const RATIOMODE_OUTBOUND = 'outbound';

	/**
	 * Width of the image in pixels
	 *
	 * @return integer
	 */
	public function getWidth();

	/**
	 * Height of the image in pixels
	 *
	 * @return integer
	 */
	public function getHeight();

	/**
	 * Edge / aspect ratio of the image
	 *
	 * @param boolean $respectOrientation If false (the default), orientation is disregarded and always a value >= 1 is returned (like usual in "4 / 3" or "16 / 9")
	 * @return float
	 */
	public function getAspectRatio($respectOrientation = FALSE);

	/**
	 * Orientation of this image, i.e. portrait, landscape or square
	 *
	 * @return string One of this interface's ORIENTATION_* constants.
	 */
	public function getOrientation();

	/**
	 * Whether this image is square aspect ratio and therefore has a square orientation
	 *
	 * @return boolean
	 */
	public function isOrientationSquare();

	/**
	 * Whether this image is in landscape orientation
	 *
	 * @return boolean
	 */
	public function isOrientationLandscape();

	/**
	 * Whether this image is in portrait orientation
	 *
	 * @return boolean
	 */
	public function isOrientationPortrait();

	/**
	 * One of PHPs IMAGETYPE_* constants that reflects the image type
	 *
	 * @see http://php.net/manual/image.constants.php
	 * @return integer
	 */
	public function getType();

	/**
	 * File extension of the image without leading dot.
	 *
	 * @return string
	 */
	public function getFileExtension();

	/**
	 * @param integer $maximumWidth maximum width of the thumbnail
	 * @param integer $maximumHeight maximum height of the thumbnail
	 * @return \TYPO3\Media\Domain\Model\ImageVariant
	 */
	public function getThumbnail($maximumWidth = NULL, $maximumHeight = NULL);

}
